@@ -1,3 +1,8 @@
+-- | You don't need to import this module to enable bash completion.
+--
+-- See
+-- <http://github.com/pcapriotti/optparse-applicative/wiki/Bash-Completion the wiki>
+-- for more information on bash completion.
 module Options.Applicative.BashCompletion
   ( bashCompletionParser
   ) where
@@ -23,7 +28,7 @@ bashCompletionParser pinfo pprefs = complParser
         (   bashCompletionQuery pinfo pprefs
         <$> (many . strOption) (long "bash-completion-word"
                                   `mappend` internal)
-        <*> option (long "bash-completion-index" `mappend` internal) )
+        <*> option auto (long "bash-completion-index" `mappend` internal) )
       , failure <$>
           (bashCompletionScript <$>
             strOption (long "bash-completion-script" `mappend` internal)) ]
@@ -37,7 +42,7 @@ bashCompletionQuery pinfo pprefs ws i _ = case runCompletion compl pprefs of
     list_options =
         fmap concat
       . sequence
-      . mapParser (\_ -> opt_completions)
+      . mapParser (const opt_completions)
 
     opt_completions opt = case optMain opt of
       OptReader ns _ _ -> return $ show_names ns
