@@ -10,7 +10,12 @@ module Options.Applicative.Extra (
   customExecParser,
   customExecParserMaybe,
   execParserPure,
+  getParseResult,
+  handleParseResult,
+  parserFailure,
+  renderFailure,
   ParserFailure(..),
+  overFailure,
   ParserResult(..),
   ParserPrefs(..),
   CompletionResult(..),
@@ -19,7 +24,7 @@ module Options.Applicative.Extra (
 import Control.Applicative (pure, (<$>), (<|>), (<**>))
 import Data.Monoid (mempty, mconcat)
 import System.Environment (getArgs, getProgName)
-import System.Exit (exitWith, ExitCode(..))
+import System.Exit (exitSuccess, exitWith, ExitCode(..))
 import System.IO (hPutStrLn, stderr)
 
 import Options.Applicative.BashCompletion
@@ -57,22 +62,36 @@ execParser = customExecParser (prefs idm)
 
 -- | Run a program description with custom preferences.
 customExecParser :: ParserPrefs -> ParserInfo a -> IO a
-customExecParser pprefs pinfo = do
-  args <- getArgs
-  case execParserPure pprefs pinfo args of
-    Success a -> return a
-    Failure failure -> do
+customExecParser pprefs pinfo
+  = execParserPure pprefs pinfo <$> getArgs
+  >>= handleParseResult
+
+-- | Handle `ParserResult`.
+handleParseResult :: ParserResult a -> IO a
+handleParseResult (Success a) = return a
+handleParseResult (Failure failure) = do
       progn <- getProgName
-      let (msg, exit) = execFailure failure progn
+      let (msg, exit) = renderFailure failure progn
       case exit of
         ExitSuccess -> putStrLn msg
         _           -> hPutStrLn stderr msg
       exitWith exit
-    CompletionInvoked compl -> do
+handleParseResult (CompletionInvoked compl) = do
       progn <- getProgName
       msg <- execCompletion compl progn
       putStr msg
-      exitWith ExitSuccess
+      exitSuccess
+
+-- | Extract the actual result from a `ParserResult` value.
+--
+-- This function returns 'Nothing' in case of errors.  Possible error messages
+-- or completion actions are simply discarded.
+--
+-- If you want to display error messages and invoke completion actions
+-- appropriately, use 'handleParseResult' instead.
+getParseResult :: ParserResult a -> Maybe a
+getParseResult (Success a) = Just a
+getParseResult _ = Nothing
 
 -- | Run a program description in pure code.
 --
@@ -81,16 +100,16 @@ customExecParser pprefs pinfo = do
 -- simply returns 'Nothing'.
 --
 -- If you need to keep track of error messages, use 'execParserPure' instead.
+{-# DEPRECATED execParserMaybe "Use execParserPure together with getParseResult instead" #-}
 execParserMaybe :: ParserInfo a -> [String] -> Maybe a
 execParserMaybe = customExecParserMaybe (prefs idm)
 
 -- | Run a program description with custom preferences in pure code.
 --
 -- See 'execParserMaybe' for details.
+{-# DEPRECATED customExecParserMaybe "Use execParserPure together with getParseResult instead" #-}
 customExecParserMaybe :: ParserPrefs -> ParserInfo a -> [String] -> Maybe a
-customExecParserMaybe pprefs pinfo args = case execParserPure pprefs pinfo args of
-  Success r -> Just r
-  _         -> Nothing
+customExecParserMaybe pprefs pinfo args = getParseResult $ execParserPure pprefs pinfo args
 
 -- | The most general way to run a program description in pure code.
 execParserPure :: ParserPrefs       -- ^ Global preferences for this parser
@@ -108,15 +127,20 @@ execParserPure pprefs pinfo args =
                  <|> (Right <$> infoParser pinfo) }
     p = runParserInfo pinfo' args
 
+-- | Generate a `ParserFailure` from a `ParseError` in a given `Context`.
+--
+-- This function can be used, for example, to show the help text for a parser:
+--
+-- @handleParseResult . Failure $ parserFailure pprefs pinfo ShowHelpText mempty@
 parserFailure :: ParserPrefs -> ParserInfo a
               -> ParseError -> Context
-              -> ParserFailure
+              -> ParserFailure ParserHelp
 parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
   let h = with_context ctx pinfo $ \names pinfo' -> mconcat
             [ base_help pinfo'
             , usage_help progn names pinfo'
             , error_help ]
-  in (render_help h, exit_code)
+  in (h, exit_code, prefColumns pprefs)
   where
     exit_code = case msg of
       ErrorMsg _   -> ExitFailure (infoFailureCode pinfo)
@@ -130,11 +154,6 @@ parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
     with_context NullContext i f = f [] i
     with_context (Context n i) _ f = f n i
 
-    render_help :: ParserHelp -> String
-    render_help = (`displayS` "")
-                . renderPretty 1.0 (prefColumns pprefs)
-                . helpText
-
     usage_help progn names i = case msg of
       InfoMsg _ -> mempty
       _         -> usageHelp $ vcatChunks
@@ -160,3 +179,8 @@ parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
     show_full_help = case msg of
       ShowHelpText -> True
       _            -> prefShowHelpOnError pprefs
+
+renderFailure :: ParserFailure ParserHelp -> String -> (String, ExitCode)
+renderFailure failure progn =
+  let (h, exit, cols) = execFailure failure progn
+  in (renderHelp cols h, exit)
