@@ -1,4 +1,4 @@
-{-# LANGUAGE GADTs, Rank2Types #-}
+{-# LANGUAGE Rank2Types, ExistentialQuantification #-}
 module Options.Applicative.Types (
   ParseError(..),
   ParserInfo(..),
@@ -20,10 +20,12 @@ module Options.Applicative.Types (
   CompletionResult(..),
   ParserFailure(..),
   ParserResult(..),
+  overFailure,
   Args,
   ArgPolicy(..),
   OptHelpInfo(..),
   OptTree(..),
+  ParserHelp(..),
 
   fromM,
   oneM,
@@ -42,6 +44,7 @@ import Control.Monad (ap, liftM, MonadPlus, mzero, mplus)
 import Data.Monoid (Monoid(..))
 import System.Exit (ExitCode(..))
 
+import Options.Applicative.Help.Types
 import Options.Applicative.Help.Pretty
 import Options.Applicative.Help.Chunk
 
@@ -168,12 +171,12 @@ instance Functor OptReader where
   fmap f (CmdReader cs g) = CmdReader cs ((fmap . fmap) f . g)
 
 -- | A @Parser a@ is an option parser returning a value of type 'a'.
-data Parser a where
-  NilP :: Maybe a -> Parser a
-  OptP :: Option a -> Parser a
-  MultP :: Parser (a -> b) -> Parser a -> Parser b
-  AltP :: Parser a -> Parser a -> Parser a
-  BindP :: Parser a -> (a -> Parser b) -> Parser b
+data Parser a
+  = NilP (Maybe a)
+  | OptP (Option a)
+  | forall x . MultP (Parser (x -> a)) (Parser x)
+  | AltP (Parser a) (Parser a)
+  | forall x . BindP (Parser x) (x -> Parser a)
 
 instance Functor Parser where
   fmap f (NilP x) = NilP (fmap f x)
@@ -236,14 +239,51 @@ instance Monoid Completer where
 newtype CompletionResult = CompletionResult
   { execCompletion :: String -> IO String }
 
-newtype ParserFailure = ParserFailure
-  { execFailure :: String -> (String, ExitCode) }
+instance Show CompletionResult where
+  showsPrec p _ = showParen (p > 10) $
+    showString "CompletionResult _"
+
+newtype ParserFailure h = ParserFailure
+  { execFailure :: String -> (h, ExitCode, Int) }
+
+instance Show h => Show (ParserFailure h) where
+  showsPrec p (ParserFailure f)
+    = showParen (p > 10)
+    $ showString "ParserFailure "
+    . showsPrec 11 (f "<program>")
+
+instance Functor ParserFailure where
+  fmap f (ParserFailure err) = ParserFailure $ \progn ->
+    let (h, exit, cols) = err progn in (f h, exit, cols)
 
 -- | Result of 'execParserPure'.
 data ParserResult a
   = Success a
-  | Failure ParserFailure
+  | Failure (ParserFailure ParserHelp)
   | CompletionInvoked CompletionResult
+  deriving Show
+
+instance Functor ParserResult where
+  fmap f (Success a) = Success (f a)
+  fmap _ (Failure f) = Failure f
+  fmap _ (CompletionInvoked c) = CompletionInvoked c
+
+overFailure :: (ParserHelp -> ParserHelp)
+            -> ParserResult a -> ParserResult a
+overFailure f (Failure failure) = Failure $ fmap f failure
+overFailure _ r = r
+
+instance Applicative ParserResult where
+  pure = Success
+  Success f <*> r = fmap f r
+  Failure f <*> _ = Failure f
+  CompletionInvoked c <*> _ = CompletionInvoked c
+
+instance Monad ParserResult where
+  return = pure
+  Success x >>= f = f x
+  Failure f >>= _ = Failure f
+  CompletionInvoked c >>= _ = CompletionInvoked c
 
 type Args = [String]
 
